REBOL [
	Title: "Make shortcut links to REBOL"
	Rights: "Copyright REBOL Technologies 1997-2005. All rights reserved."
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

link-rebol: func [
	"Install the shortcut links for REBOL."
	boot-path [file!]
	home-dir [file!]
	product [string!] "Product name"
	options [block!] "Possible options: create-group desktop auto-start"
	/local programs desktop progs result linkname note startup
][
	result: true
	linkname: rejoin ["REBOL " product ".lnk"]
	note: rejoin ["REBOL " product " Shortcut"]
	boot-path: to-local-file boot-path
	home-dir: to-local-file home-dir

	;-- Get programs and desktop locations:
	if exists-reg? ms-folders [
		programs: get-reg ms-folders "Programs"
		desktop:  get-reg ms-folders "Desktop"
		startup:  get-reg ms-folders "Startup"
	]

	;-- Make file link to desktop:
	if find options 'desktop [
		if not all [
			desktop
			create-link/start/note boot-path join desktop ["\" linkname] home-dir note
			;probe "Create ICON"
		] [result: false]
	]

	;-- Make the program group:
	if find options 'create-group [
		if not all [
			programs
			exists? progs: dirize to-rebol-file programs
			any [exists? progs: dirize progs/REBOL attempt [make-dir/deep progs]]
			create-link/start/note boot-path join programs ["\REBOL\" linkname] home-dir note
			;probe "Create LINKS"
		] [result: false]
	]

	;-- Auto-start application:
	if find options 'auto-start [
		if not all [
			startup
			exists? dirize to-rebol-file startup
			create-link/start/note boot-path join startup ["\" linkname] home-dir note
		] [result: false]
	]

	result
]

unlink-rebol: func [
	; Remove shortcut links created for installation
	product [string!] "Product name"
	/local linkname programs desktop startup
][
	linkname: rejoin ["REBOL " product ".lnk"]

	;-- Get programs and desktop locations:
	if exists-reg? ms-folders [
		programs: to-rebol-file get-reg ms-folders "Programs"
		desktop:  to-rebol-file get-reg ms-folders "Desktop"
		startup:  to-rebol-file get-reg ms-folders "Startup"
	]

	if exists? desktop/:linkname [
		attempt [delete desktop/:linkname]
	]

	if exists? programs/REBOL/:linkname [
        ; should delete whole dir recursively?
        attempt [delete programs/REBOL/:linkname]
    ]

    if exists? startup/:linkname [
        attempt [delete startup/:linkname]
    ]
]

default-prog-dir?: does [
	; Default program dir for REBOL program
	either winos? [ ; windows
		lowercase repend dirize to-rebol-file any [
			get-reg/hklm ms-base "ProgramFilesDir"
			"c:\program files"
		]["rebol/" system/product "/"]
	][
		system/options/home
	]
]

default-data-dir?: has [dir] [
	; Default app dir for storing public cache
	either winos? [ ; windows
		dir: get-reg ms-folders "Appdata"
		if not dir [return default-prog-dir?]
		lowercase append dirize to-rebol-file dir "rebol/"
	][
		join get-modes %~/ 'full-path pick [
			%"Library/Application Support/rebol/view/"
			%.rebol/view/
		] system/version/4 = 2
	]
]

;do %utils.r
;print default-prog-dir?
;print default-data-dir?
;halt
