REBOL [
	Title:  "REBOL Protocols: Header Processing"
	Author: "Carl Sassenrath"
	Rights: "Copyright REBOL Technologies 1997-2004"
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]
; Major changes 21-Jan-04 from Romano Paolo Tenca 

header-rules: make object! [
	;-- Break-up header into a list of fields and string values.
	template: w: set-word: head-list: spot: full-line: line: none
	invalid: [] ;to collect invalid header lines
	spaces: charset " ^-"
	leader: charset " ^-^/^m" ; ignored at start of header
	field-chars: charset [#"a" - #"z" #"A" - #"Z" #"0" - #"9" "-_."]
	; could be also: "!#$%&'*+^^`|~" (RFC)
	; #!$% are not possible in REBOL at the start of a word
	new-line: [copy line to newline newline] 
	field-value: [
		(full-line: make string! 32)
		new-line (all [line insert tail full-line trim line])
		any [
			some spaces new-line (insert insert tail full-line " " trim any [line ""])
		]
	]
	message-header: [
		copy w some field-chars ":" field-value (
			w: to-word w
			set-word: to-set-word w
			; fixes same header appearing multiple times (RAMBO #3090)
			either spot: find head-list set-word [
				either block? spot/2 [
					insert tail spot/2 full-line
				] [
					spot/2: reduce [spot/2 full-line]
				]
			] [
				insert insert tail head-list set-word full-line
			]
		)
	]
	content: [
		full-line: (insert insert tail head-list to-set-word "content" full-line)
		to end
	]
	message: [
		;content could not be present in data, so we test for end
		(head-list: make block! 10)
		any leader
		some [
			message-header |
			newline content break |
			copy line thru newline (insert tail invalid line)
		]
	]
	set 'parse-header parse-head: func [
		{Returns a header object with header fields and their values}
		parent [object! none!] "Default header object"
		data [any-string!] "String to parse"
		/multiple {Obsolete. Here for compatibility only.}
	][
		clear invalid
		template: parent
		any [
			parse/all data message
			net-error "Headers not correctly parsed" 
		]
		make either parent [parent] [object!] head-list
	]
]

parse-header-date: func [
	data [string! none!] /local despace here date count
][
	if none? data [return none]
	count: 0
	despace: [to " " here: (
			change here "/"
			either count = 3 [if #"-" <> second here [insert next here #"+"]]
			[count: count + 1]) despace | none]
	parse/all data [opt [thru "," thru " "] date: despace]
	if error? data: try [to-date date] [data: now]
	data
]
