REBOL [
	Title:  "REBOL Protocols: NNTP"
	Author: "Jeff Kreis"
	Rights: "Copyright REBOL Technologies 1997-2002"
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]

make Root-Protocol [
	"REBOL Network News Port."

	scheme: 'nntp
	port-id: 119
	port-flags: system/standard/port-flags/pass-thru
	
	open-check:      [none    "20"]  
	fill-check:      [none    "2"]  
	close-check:     ["QUIT" "205"]
	group-check:     [reform ["GROUP" port/locals/newsgroup]  "211"]
	list-check:      ["LIST" "215"]
	listgroup-check: [none   "211"]
	article-check:   [reform ["ARTICLE" port/locals/message]  "220"]
	post-check:      ["POST"   "3"]  
	done-post-check: ["."      "2"]
	authorize-check: [none    "28"]

	open: func [
		port
		/local capstring
	][
		howdey: open-proto port
		capstring: caps port/sub-port
		if find capstring "MODE" [
			howdey: mode-reader port 
			capstring: caps port/sub-port
		]

		if find/match capstring "480" [
			authenticate port
			capstring: caps port/sub-port

		]
	
		port/locals: make object! [
			newsgroup: none
			message: none
			message-block: make block! 10000
			group-block:   make block! 10000
			result: make block! 10000 
			zero-articles: false
			xhdr: found? find capstring "xhdr"
			listgroup: found? find capstring "listgroup"
			where-I-was: 0
		]
	
		port/state/flags: port/state/flags or port-flags
		port/state/tail:  1000000 ;-- count it up later
		port/state/index: 0 

		what-gives port
	] 
	
	what-gives: func [port][
		port/locals/newsgroup: any [port/path port/target]
		port/locals/newsgroup: either port/locals/newsgroup [
		first parse port/locals/newsgroup "/"][none]
		port/locals/message: port/target
		if port/locals/message = port/locals/newsgroup [
			port/locals/message: none
		]
	]
	
	copy: func [
		{ Insert takes a block of a dialect the news 
		port and then has it interpreted and executed }
		port  [port!]  "The port"
		data "result"
		/local tokens total-result temp-toke name
	][ 
		port/locals/result: make block! 1000
		any [
			all [port/locals/message port/locals/newsgroup 
				go-group port get-data port true]
			all [port/locals/newsgroup go-group port get-data port true]
			all [port/host get-groups port]
		]
		data: either any [none? port/locals/result 
			empty? port/locals/result]
			[none][copy* port/locals/result]
		port/locals/result: none
		data
	]
	
	caps: func [
		"Find out what the server can do."
		port
	][
		system/words/insert port "HELP"
		either find/match newsCaps: system/words/pick port 1 "100"  [
		    newsCaps: read-message port copy* "" 
		][newsCaps]
	]	

	authenticate: func [
		"Authenticate ourselves to the server"
		port [port!] 
	][
		if none? port/pass [
			net-error "Password required"
		]
		system/words/insert port/sub-port rejoin [
			"AUTHINFO USER " port/user
		]

		system/words/pick port/sub-port 1 ;-- gobble the more auth

		system/words/insert port/sub-port rejoin [
			"AUTHINFO PASS " port/pass
		]
		net-utils/confirm port/sub-port authorize-check


	]	

	mode-reader: func [
		"Some servers may require you to go mode reader first"
		port [port!]
	][
		system/words/insert port/sub-port "MODE READER"
		net-utils/confirm port/sub-port open-check
	]	 	

	read-message: func [
		"Read a message from the NEWS server"
		port [port!]
		buf [string!]
		/local line
	][
		while [(line: system/words/pick port 1) <> "."] [
			system/words/insert tail buf line
			system/words/insert tail buf newline
		]
		buf
	]	

	go-group: func [
		"Enter into a newsgroup"
		port 
		/local response msg-cnt
	][
		clear port/locals/message-block
		response: load net-utils/confirm port/sub-port reduce bind group-check 'port
		if response/2 = 0 [port/locals/zero-articles: true return true]
		where-I-was: 0
		port/state/tail: 1 + response/4 - response/3 
		any [
			all [port/locals/xhdr fill-message-block port rejoin [
				copy* "XHDR MESSAGE-ID " 
				response/3 "-" response/4] 2 true
			]
			all [port/locals/listgroup 
				fill-mesage-block port "LISTGROUP" 1 true]
		]   
		true
	]	

	fill-message-block: func [port command index][
		system/words/insert port/sub-port command 
		net-utils/confirm port/sub-port fill-check 
		while [(line: system/words/pick port/sub-port 1) <> "."] [
			system/words/insert/only tail port/locals/message-block system/words/pick parse line none index
		]
		port/state/tail: length? port/locals/message-block	  
	] 	

	cool-response: func [port][none? find/match system/words/pick port/sub-port 1 "4"] ;-- 4's error	
	Grab-article: func [port][
		system/words/insert port/sub-port "ARTICLE"
		either cool-response port [
			system/words/insert/only tail port/locals/result 
				read-message port/sub-port copy* ""
		][false]
	]	

	go-next: func [port][
		system/words/insert port/sub-port "NEXT"
		cool-response port
	]	

	stat: func [port where][
		system/words/insert port/sub-port append copy* "STAT " where
		system/words/pick port/sub-port 1
	]	

	get-an-article: func [port][		    
		system/words/insert port/sub-port append copy* "ARTICLE " port/locals/message
		either cool-response port [append port/locals/result read-message port/sub-port copy* ""
		][port/locals/result: none]
	]	

	catch-up: func [port][
		while [port/locals/where-I-was < port/state/index][
			either cool-response port mess: go-next port [
				port/state/where-I-was: port/state/where-I-was + 1
				append port/locals/message-block system/words/pick parse mess none 3
			][	
				append port/locals/result none return false
			]
		]
	]	

	get-data: func [
		"Gets data from the server"
		port [port!] "The entire port, please"
		/local first-time prev-filt
	][	
		if port/locals/zero-articles [
			system/words/insert tail port/locals/result none exit
		] ;-- No articles to get here...
		
		first-time: true
		
		either port/locals/message [
			get-an-article port
			return true
		][	
			if not grab-article port [return true]
	
			while [on] [ 
				if not go-next port [return true]
				grab-article port		  
			]
		]
	]	

	pick: func [port /local result][
		port/locals/result: make block! 1000
		either none? port/locals/newsgroup [
			get-groups port
			port/locals/result: system/words/pick port/locals/result port/state/index + 1
		][	
			if empty? port/locals/message-block [
				go-group port
				if all [empty? port/locals/message-block not find/match result: stat port "" "420"] [
					append port/locals/message-block system/words/pick parse result none 3
					if catch-up port [
						grab-article port
					]
				]
			]
	
			if port/state/index <= length? port/locals/message-block [
				port/locals/message: system/words/pick port/locals/message-block port/state/index + 1
				port/locals/where-I-was: port/state/index
				get-an-article port
			]		
			if port/state/index > length? port/locals/message-block [
				port/locals/where-I-was: length? port/locals/message-block
				if all [not empty? port/locals/message-block 
					not find/match result: stat port last port/locals/message-block "420" catch-up port][
					grab-article port
				]
			]
		
		]
		result: either any [none? port/locals/result 
			empty? port/locals/result] [none][copy* port/locals/result]
		port/locals/result: none
		result
	]	

	get-groups: func [
		"Retrieve the list of newsgroups"
		port [port!] "Entire port, please"
	][
		net-utils/confirm port/sub-port list-check 
		while [(line: system/words/pick port/sub-port 1) <> "."] [
			system/words/insert tail port/locals/result first parse line none
		]
		port/state/tail: length? port/locals/result
		port/locals/result
	]	

	insert: func [
		"Post to Usenet"
		port
		data
	][
		if port/locals/newsgroup [go-group port]
		net-utils/confirm port/sub-port post-check
		system/words/insert port/sub-port data
		net-utils/confirm port/sub-port done-post-check
	]
		
	;--- Register ourselves. 
	net-utils/net-install NNTP self 119
]
